import os
import json
import shutil
import uuid
from .sdms import Sdms
from datetime import datetime
import time
from operator import itemgetter
from .execptions import *
from .model import *
import logging

logger = logging.getLogger(__name__)


class SdmsLocal(Sdms):
    def __init__(self, client, base_path):
        self.base_path = base_path

    def _add_characterization_preare(
        self,
        uploader_user_id: int,
        owner_id: int,
        char_techs: str,
        sample_id: str,
        name: str,
        type: str,
        allowed_suffixes=["xlsx", "pdf", "csv", "xls"],
        suffix="",
        description="",
    ):
        # 构建JSON数据
        data = {
            "uploader_user_id": uploader_user_id,
            "owner_id": owner_id,
            "char_techs": char_techs,
            "sample_id": sample_id,
            "name": name,
            "type": type,
            "allowed_suffixes": allowed_suffixes,
            "file_suffix": suffix,
            "description": description,
            "uuid": str(uuid.uuid4()),
            "file_path": f"{self.base_path}/raw_data/{owner_id}/{name}.{suffix}",
            "create_timestamp": int(time.time()),
            "create_time": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        }
        char_meta = CharMeta(**data)
        # 确保目录存在
        os.makedirs(f"{self.base_path}/raw_data/{owner_id}", exist_ok=True)
        os.makedirs(f"{self.base_path}/meta/{owner_id}", exist_ok=True)

        # 原数据
        meta_file_path = self._get_meta_file_path(owner_id, name)

        return char_meta, meta_file_path

    def add_characterization_buffer(
        self,
        uploader_user_id: int,
        owner_id: int,
        char_techs: str,
        sample_id: str,
        name: str,
        type: str,
        allowed_suffixes=["xlsx", "pdf", "csv", "xls"],
        description="",
        buffer="",
        suffix="",
    ):

        try:
            # 检查文件后缀
            res = self._check_file_suffix("", suffix, allowed_suffixes)
            if not res[1]:
                logger.error(
                    f"add | user: {owner_id} | {name} | 不支持的文件后缀: {suffix}"
                )
                raise SdmsInvalidFileFormatError(f"不支持的文件后缀: {suffix}")

            data, meta_file_path = self._add_characterization_preare(
                uploader_user_id,
                owner_id,
                char_techs,
                sample_id,
                name,
                type,
                allowed_suffixes,
                res[0],
                description,
            )

            # 如果原数据已存在，则返回错误
            if os.path.exists(meta_file_path):
                logger.error(
                    f"add | user: {owner_id} | {name} | 文件已存在: {os.path.basename(meta_file_path)}"
                )
                raise SdmsFileDuplicateError(
                    f"文件已存在: {os.path.basename(meta_file_path)}"
                )

            with open(meta_file_path, "w") as f:
                json.dump(data.to_dict(), f, indent=4)

            # 写入文件
            # 检查数据类型
            if isinstance(buffer, str):
                # 如果是字符串，使用文本写入模式
                with open(data.file_path, "w", encoding="utf-8") as file:
                    file.write(buffer)
            elif isinstance(buffer, bytes):
                # 如果是二进制数据，使用二进制写入模式
                with open(data.file_path, "wb") as file:
                    file.write(buffer)
            else:
                raise ValueError("不支持的数据类型。数据必须是字符串或二进制。")

            logger.info(f"add | user: {owner_id} | {name} | 上传成功")
            return os.path.basename(meta_file_path)
        except SdmsException as e:
            raise e
        except Exception as e:
            logger.error(f"add | user: {owner_id} | {name} | 异常: {e}")
            raise SdmsUnknownError(str(e))

    def add_characterization_file(
        self,
        uploader_user_id: int,
        owner_id: int,
        char_techs: str,
        sample_id: str,
        name: str,
        type: str,
        allowed_suffixes=["xlsx", "pdf", "csv", "xls"],
        description="",
        temp_file_path="",
        suffix="",
    ):

        try:
            # 检查文件后缀
            res = self._check_file_suffix(temp_file_path, suffix, allowed_suffixes)
            if not res[1]:
                logger.error(
                    f"add | user: {owner_id} | {name} | 不支持的文件后缀: {suffix}"
                )
                raise SdmsInvalidFileFormatError(f"不支持的文件后缀: {suffix}")

            # 确保临时文件存在
            if not os.path.exists(temp_file_path):
                logger.error(
                    f"add | user: {owner_id} | {name} | 临时文件不存在: {temp_file_path}"
                )
                raise SdmsFileNotFoundError(f"临时文件不存在: {temp_file_path}")

            data, meta_file_path = self._add_characterization_preare(
                uploader_user_id,
                owner_id,
                char_techs,
                sample_id,
                name,
                type,
                allowed_suffixes,
                res[0],
                description,
            )

            # 原数据
            meta_file_path = self._get_meta_file_path(owner_id, name)

            # 如果原数据已存在，则返回错误
            if os.path.exists(meta_file_path):
                logger.error(
                    f"add | user: {owner_id} | {name} | 文件已存在: {os.path.basename(meta_file_path)}"
                )
                raise SdmsFileDuplicateError(
                    f"文件已存在: {os.path.basename(meta_file_path)}"
                )

            with open(meta_file_path, "w") as f:
                json.dump(data.to_dict(), f, indent=4)

            # 复制文件
            shutil.copy2(temp_file_path, data.file_path)
            logger.info(f"add | user: {owner_id} | {name} | 上传成功")
            return os.path.basename(meta_file_path)

        except SdmsException as e:
            raise e
        except Exception as e:
            logger.error(f"add | user: {owner_id} |  {name} | 异常: {e}")
            raise SdmsUnknownError(str(e))

    def delete_characterization_file(self, user_id, name):

        try:
            # 文件不存在, 直接返回成功
            meta_file_path = self._get_meta_file_path(user_id, name)
            if not os.path.exists(meta_file_path):
                logger.info(f"delete | user: {user_id} | {name} | 文件不存在")
                return True

            # 读取JSON文件
            with open(meta_file_path, "r") as f:
                data = json.load(f)
            # 删除原始数据文件
            os.remove(data["file_path"])
            # 删除JSON文件
            os.remove(meta_file_path)

            logger.info(f"delete | user: {user_id} | {name} | 删除成功")
            return True
        except Exception as e:
            logger.error(f"delete | user: {user_id} | {name} | 异常: {e}")
            return False

    def query_characterization_files(self, user_id, key_name):

        try:
            # 获取用户目录下的所有文件
            meta_dir = f"{self.base_path}/meta/{user_id}"
            if not os.path.exists(meta_dir):
                logger.error(f"query | user: {user_id} | {key_name} | 文件列表为空")
                return []

            files = os.listdir(meta_dir)
            result = []
            for file in files:
                if key_name in file or key_name is None or key_name == "":
                    with open(f"{meta_dir}/{file}", "r") as f:
                        data = json.load(f)
                        result.append(CharMeta.from_json(data))

            logger.info(f"query | user: {user_id} | {key_name} | 查询成功")
            return result
        except Exception as e:
            logger.error(f"query | user: {user_id} | {key_name} | 异常: {e}")
            return []

    def get_characterization_file(self, owner_id, name):

        try:
            # 读取JSON文件
            meta_file_path = self._get_meta_file_path(owner_id, name)
            with open(meta_file_path, "r") as f:
                data = json.load(f)

            meta_data = CharMeta.from_json(data)
            # 判断文件是否存在
            raw_data_path = meta_data.file_path

            if not os.path.exists(raw_data_path):
                logger.error(f"get | user: {owner_id} | {name} | 文件不存在")
                return None

            # 读取原始数据文件
            raw_bytes = self.read_in_chunks_bytes(raw_data_path)

            return CharContext(meta_data, raw_bytes)
        except Exception as e:
            logger.error(f"get | user: {owner_id} | {name} | 异常: {e}")
            return None

    def result_add(self, owner_id: int, name: str, context: str):

        result_file_path = self._get_result_file_path(owner_id, name)
        if os.path.exists(result_file_path):
            logger.error(f"result add | user: {owner_id} | {name} | 文件已存在")
            raise SdmsFileDuplicateError(
                f"文件已存在: {os.path.basename(result_file_path)}"
            )

        timestamp = int(time.time())
        yyyymmdd = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        # 构建JSON数据
        data = {
            "owner_id": owner_id,
            "name": name,
            "result": context,
            "create_timestamp": timestamp,
            "create_time": yyyymmdd,
            "update_timestamp": timestamp,
            "update_time": yyyymmdd,
        }
        result_meta = ResultMeta(**data)

        # 确保目录存在
        os.makedirs(f"{self.base_path}/result/{owner_id}", exist_ok=True)

        try:
            # 写入文件
            with open(result_file_path, "w", encoding="utf-8") as file:
                json.dump(result_meta.to_dict(), file, indent=4)

            logger.info(f"result add | user: {owner_id} | {name} | 上传成功")
            return os.path.basename(result_file_path)
        except Exception as e:
            logger.error(f"result add | user: {owner_id} | {name} | 异常: {e}")
            raise SdmsUnknownError(str(e))

    def result_put(self, owner_id: int, name: str, context: str):

        result_file_path = self._get_result_file_path(owner_id, name)
        if not os.path.exists(result_file_path):
            logger.error(f"result put | user: {owner_id} | {name} | 文件不存在")
            raise SdmsFileNotFoundError(
                f"文件不存在: {os.path.basename(result_file_path)}"
            )

        # 读取现有数据
        datastr = self.read_in_chunks(result_file_path)
        result_data = ResultMeta.from_json(datastr)
        result_data.update_timestamp = int(time.time())
        result_data.update_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        result_data.result = context

        try:
            # 写入文件
            with open(result_file_path, "w", encoding="utf-8") as file:
                json.dump(result_data.to_dict(), file, indent=4)

            logger.info(f"result put | user: {owner_id} | {name} | 更新成功")
            return os.path.basename(result_file_path)
        except Exception as e:
            logger.error(f"result put | user: {owner_id} | {name} | 异常: {e}")
            raise SdmsUnknownError(str(e))

    def result_delete(self, owner_id: int, name: str):

        result_file_path = self._get_result_file_path(owner_id, name)
        if not os.path.exists(result_file_path):
            logger.error(f"result delete | user: {owner_id} | {name} | 文件不存在")
            return False

        try:
            os.remove(result_file_path)
            logger.info(f"result delete | user: {owner_id} | {name} | 删除成功")
            return True
        except Exception as e:
            logger.error(f"result delete | user: {owner_id} | {name} | 异常: {e}")
            return False

    def result_query(self, owner_id: int, key_name: str):

        try:
            # 获取用户目录下的所有文件
            result_dir = f"{self.base_path}/result/{owner_id}"
            if not os.path.exists(result_dir):
                logger.error(f"query | user: {owner_id} | {key_name} | 文件列表为空")
                return []

            # 获取文件列表并添加修改时间
            files = []
            for file in os.listdir(result_dir):
                if key_name in file or key_name is None or key_name == "":
                    file_path = os.path.join(result_dir, file)
                    update_time = os.path.getmtime(file_path)
                    files.append((file, update_time))
            # 按修改时间倒序排序
            files.sort(key=itemgetter(1), reverse=True)
            result = []
            # 剔除文件后缀
            for file, _ in files:
                result.append(os.path.splitext(file)[0])

            return result
        except Exception as e:
            logger.error(f"query | user: {owner_id} | {key_name} | 异常: {e}")
            return []

    def result_get(self, owner_id: int, name: str):
        try:
            result_file_path = self._get_result_file_path(owner_id, name)
            if not os.path.exists(result_file_path):
                logger.error(f"result get | user: {owner_id} | {name} | 文件不存在")
                return None

            data = self.read_in_chunks(result_file_path)

            return ResultMeta.from_json(data)
        except Exception as e:
            logger.error(f"result get | user: {owner_id} | {name} | 异常: {e}")
            return None
